#include <u.h>   
#include <libc.h>
#include <bio.h>
#include <libsec.h>
#include <mp.h>

#pragma	varargck	type	"M"	uchar*

static int
digestfmt(Fmt *fmt)
{
	char buf[SHA2_256dlen*2+1];
	uchar *p;
	int i;

	p = va_arg(fmt->args, uchar*);
	for(i=0; i<SHA2_256dlen; i++)
		sprint(buf+2*i, "%.2ux", p[i]);
	return fmtstrcpy(fmt, buf);
}

/* hashrange takes a given input string and parameters that determine how large a nonce to use, then generates every possible string with a nonce of the given size and calculates its sha256 hash, searching for the lowest hash and printing each lowest as it finds it */

static void
hashrange(uchar *recbuf, uint cstart, uint cend, uint buflen, int offset, int padding)
{
	uchar digest[SHA2_256dlen];
	int thisloc;
	mpint *lowest;
	mpint *current;

	memset(recbuf + offset, cstart, (buflen - offset) - padding);
	recbuf[buflen -1] = 0;
	thisloc = (buflen - 1) - padding;

	sha2_256(recbuf, strlen((char*)recbuf), digest, nil);
	lowest = betomp(digest, SHA2_256dlen, nil);
	current = betomp(digest, SHA2_256dlen, nil);
	print("%M\t%s\n", digest, (char*)recbuf);

	while (thisloc >= offset){
		sha2_256(recbuf, strlen((char*)recbuf), digest, nil);
		betomp(digest, SHA2_256dlen, current);
		if(mpcmp(current, lowest) == -1){
			mpassign(current, lowest);
			print("%M\t%s\n", digest, (char*)recbuf);
		}
		thisloc = (buflen - 1) - padding;
		while (++recbuf[thisloc] == cend){
			recbuf[thisloc] = cstart;
			thisloc--;
		}
	}
}

void
main(int argc, char *argv[])
{
	uchar *startstring;
	uint charstart, charend, bufferlen;
	int baselen, actlen, siglen, noncelen;
	char *baseblock, *action, *signature;

	fmtinstall('M', digestfmt);

	charstart = 97;
	charend = 123;

	if(argc == 1){
		argv[1] = strdup("531a5e2d5ae04b1a17e75b7a700c3eb0");
		argv[2] = strdup("ABCXYZ");
		argv[3] = strdup("b9841310602168840721eca477e28aa7");
		argv[4] = strdup("5");
		argc = 5;
	}
	
	if(argc < 5)
		exits(nil);

	baseblock = argv[1];
	action = argv[2];
	signature = argv[3];
	noncelen = atoi(argv[4]);
	
	baselen = strlen(baseblock);
	actlen = strlen(action);
	siglen = strlen(signature);
	bufferlen = baselen + actlen + siglen + noncelen + 1024;

	startstring=(uchar*)malloc(bufferlen);
	sprint((char*)startstring, "%s-%s-%s-", baseblock, action, signature);
	bufferlen=strlen((char*)startstring) + noncelen +2;
//	print("bufferlen = %d offset = %ld\n", bufferlen, strlen((char*)startstring));
	hashrange(startstring, charstart, charend, bufferlen, strlen((char*)startstring), 2);

	exits(nil);
}

/* incorporates code taken from the plan 9 rsa libs and auth utilities */

/*
	int lowhash;
	int lowcheck;
	int lowdigit;
	lowhash = 0;
	lowcheck = 0;
	lowdigit = 0;

		while((int)digest[lowcheck] == 0)
			lowcheck++;
		if(lowcheck == lowhash){
			if((int)digest[lowcheck] < lowdigit){
				lowdigit = (int)digest[lowcheck];
				print("%M\t%s\n", digest, (char*)recbuf);
			}
		}
		if(lowcheck > lowhash){
			lowhash = lowcheck;	
			lowdigit = (int)digest[lowcheck];
			print("%M\t%s\n", digest, (char*)recbuf);
		}
		lowcheck = 0;
*/

/*
	uchar lowest[SHA2_256dlen];
	memcpy(lowest, digest, SHA2_256dlen);
		if(strncmp((char*)digest, (char*)lowest, SHA2_256dlen) == -1){
			memcpy(lowest, digest, SHA2_256dlen);
			print("%M\t%s\n", digest, (char*)recbuf);
		}
*/
